"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var defaultOptions = {
    silent: true,
    preserveEmptyAssets: false,
};
var WebpackRemoveEmptyJSChunksPlugin = /** @class */ (function () {
    function WebpackRemoveEmptyJSChunksPlugin(options) {
        this._name = 'webpack-remove-empty-js-chunks-plugin';
        this._options = Object.assign({}, defaultOptions, options);
        this._excludeChunks = {};
    }
    WebpackRemoveEmptyJSChunksPlugin.prototype.apply = function (compiler) {
        var _this = this;
        compiler.hooks.thisCompilation.tap(this._name, function (compilation) {
            compilation.mainTemplate.hooks.require.tap(_this._name, function () {
                compilation.mainTemplate.hooks.requireEnsure.tap(_this._name, function (source) {
                    compilation.chunks.forEach(function (chunk) {
                        var chunkModules = chunk.getModules();
                        if (!chunkModules.length) {
                            return;
                        }
                        var isMixedModulesChunk = chunk.getModules()
                            .some(function (module) { return module.type !== 'css/mini-extract'; });
                        if (!isMixedModulesChunk) {
                            _this._excludeChunks[chunk.id] = chunk.renderedHash;
                            if (!_this._options.silent) {
                                // eslint-disable-next-line no-console
                                console.log(_this._name + ': excluding ' + chunk.id + '.' + chunk.renderedHash + '.js' + ', because it\'s empty');
                            }
                        }
                    });
                    var strToSearch = 'if(installedChunkData !== 0) { // 0 means "already installed".';
                    var strParts = source.split(strToSearch);
                    // we may have already updated the source
                    if (strParts.length === 1) {
                        return source;
                    }
                    var excludeChunksFiles = {};
                    Object.keys(_this._excludeChunks).forEach(function (chunkId) { excludeChunksFiles[chunkId] = 1; });
                    return strParts[0] +
                        '\n' + 'var excludeFiles = ' + JSON.stringify(excludeChunksFiles) +
                        '\n' + 'if(installedChunkData !== 0 && !excludeFiles[chunkId]) { // 0 means "already installed".' +
                        strParts[1];
                });
            });
        });
        if (!this._options.preserveEmptyAssets) {
            compiler.hooks.emit.tap(this._name, function (compilation) {
                Object.keys(_this._excludeChunks).forEach(function (chunkToExclude) {
                    var chunkFileName = chunkToExclude + '.' + _this._excludeChunks[chunkToExclude] + '.js';
                    delete compilation.assets[chunkFileName];
                });
            });
        }
    };
    return WebpackRemoveEmptyJSChunksPlugin;
}());
exports.WebpackRemoveEmptyJSChunksPlugin = WebpackRemoveEmptyJSChunksPlugin;
